@@ -894,7 +894,11 @@ static TermKeyResult peekkey_mouse(TermKey *tk, TermKeyKey *key, size_t *nbytep)
   key->code.mouse[0] = CHARAT(0) - 0x20;
   key->code.mouse[1] = CHARAT(1) - 0x20;
   key->code.mouse[2] = CHARAT(2) - 0x20;
-  key->modifiers     = 0;
+  key->code.mouse[3] = 0;
+
+  key->modifiers     = (key->code.mouse[0] & 0x1c) >> 2;
+  key->code.mouse[0] &= ~0x1c;
+
   *nbytep = 3;
   return TERMKEY_RES_KEY;
 }
@@ -907,11 +911,7 @@ TermKeyResult termkey_interpret_mouse(TermKey *tk, const TermKeyKey *key, TermKe
   if(button)
     *button = 0;
 
-  if(col)
-    *col  = (unsigned char)key->code.mouse[1];
-
-  if(line)
-    *line = (unsigned char)key->code.mouse[2];
+  termkey_key_get_linecol(key, line, col);
 
   if(!event)
     return TERMKEY_RES_KEY;
@@ -922,12 +922,14 @@ TermKeyResult termkey_interpret_mouse(TermKey *tk, const TermKeyKey *key, TermKe
 
   int drag = code & 0x20;
 
-  switch(code & ~0x20) {
+  code &= ~0x3c;
+
+  switch(code) {
   case 0:
   case 1:
   case 2:
     *event = drag ? TERMKEY_MOUSE_DRAG : TERMKEY_MOUSE_PRESS;
-    btn = (code & ~0x20) + 1;
+    btn = code + 1;
     break;
 
   case 3:
@@ -938,7 +940,7 @@ TermKeyResult termkey_interpret_mouse(TermKey *tk, const TermKeyKey *key, TermKe
   case 64:
   case 65:
     *event = drag ? TERMKEY_MOUSE_DRAG : TERMKEY_MOUSE_PRESS;
-    btn = (code & ~0x20) + 4 - 64;
+    btn = code + 4 - 64;
     break;
 
   default:
@@ -948,6 +950,19 @@ TermKeyResult termkey_interpret_mouse(TermKey *tk, const TermKeyKey *key, TermKe
   if(button)
     *button = btn;
 
+  if(key->code.mouse[3] & 0x80)
+    *event = TERMKEY_MOUSE_RELEASE;
+
+  return TERMKEY_RES_KEY;
+}
+
+TermKeyResult termkey_interpret_position(TermKey *tk, const TermKeyKey *key, int *line, int *col)
+{
+  if(key->type != TERMKEY_TYPE_POSITION)
+    return TERMKEY_RES_NONE;
+
+  termkey_key_get_linecol(key, line, col);
+
   return TERMKEY_RES_KEY;
 }
 
@@ -1292,6 +1307,9 @@ size_t termkey_strfkey(TermKey *tk, char *buffer, size_t len, TermKeyKey *key, T
       }
     }
     break;
+  case TERMKEY_TYPE_POSITION:
+    l = snprintf(buffer + pos, len - pos, "Position");
+    break;
   }
 
   if(l <= 0) return pos;
@@ -1389,18 +1407,32 @@ int termkey_keycmp(TermKey *tk, const TermKeyKey *key1p, const TermKeyKey *key2p
     case TERMKEY_TYPE_UNICODE:
       if(key1.code.codepoint != key2.code.codepoint)
         return key1.code.codepoint - key2.code.codepoint;
+      break;
     case TERMKEY_TYPE_KEYSYM:
       if(key1.code.sym != key2.code.sym)
         return key1.code.sym - key2.code.sym;
+      break;
     case TERMKEY_TYPE_FUNCTION:
       if(key1.code.number != key2.code.number)
         return key1.code.number - key2.code.number;
+      break;
     case TERMKEY_TYPE_MOUSE:
       {
         int cmp = strncmp(key1.code.mouse, key2.code.mouse, 4);
         if(cmp != 0)
           return cmp;
       }
+      break;
+    case TERMKEY_TYPE_POSITION:
+      {
+        int line1, col1, line2, col2;
+        termkey_interpret_position(tk, &key1, &line1, &col1);
+        termkey_interpret_position(tk, &key2, &line2, &col2);
+        if(line1 != line2)
+          return line1 - line2;
+        return col1 - col2;
+      }
+      break;
   }
 
   return key1.modifiers - key2.modifiers;
